
;;;: ---------------------- Logarithmic Math ------------------------

;;; logarithm with arbitrary base
;;
;; NOTE: The logarithm to base 1 is mathematically undefined and
;;       causes an "division by zero" error. This is not a bug.
;;
;; logN(number) = log(number)/log(N)
;;
(defun logn (base number)
  (/ (log (float number)) (log (float base))))

;;; logarithm to base 2 [pitch]
;;
(defun log2 (number)
  (/ (log (float number)) (log 2.0)))

;;; logarithm to base 10 [decibels]
;;
(defun log10 (number)
  (/ (log (float number)) (log 10.0)))

;;; exponentiantion with arbitrary base
;;
;; expN(number) = exp(log(number)*N)
;;
(defun expn (base number)
  (exp (* (log (float base)) number)))

;; with numbers it's faster to use XLISP's "expt" function
;;
(defun expn (base number)
  (expt (float base) (float number)))

;; n-th root of a number
;;
;; rootN(number) = exp(log(number)/N)
;;
(defun rootn (degree number)
  (exp (/ (log (float number)) degree)))

;; 12th root of 2 [semitone distance]
;;
(setq *semitone* (exp (/ (log 2.0) 12.0)))

;;;; --------------------------- Decibels ----------------------------

;; decibels (amplitude or volume) are based on log10

;; see http://en.wikipedia.org/wiki/Decibel

;;; linear-to-decibel conversion
;;
;; NOTE: The function signals an "division by zero" error if the
;;       "reference" is zero. This is correct because a signal
;;       cannot be compared to nothing.
;;
;; db = 20*log10(signal/reference)  [general formula]
;;
;; db = 20*(log(signal/reference)/log(10))  [using XLISP "log"]
;;
(defun decibel (reference signal)
  (* 20 (/ (log (/ (float signal) reference))) (log 10.0)))

;;; lin-to-db - same as Nyquist's linear-to-db [but slower]
;;
;; reference = 1.0        [= soundcard limit, defined by Nyquist]
;; db = 20*log10(signal)  [valid in Nyquist only]
;;
;; db = 20*(log(signal)/log(10))  [using XLISP "log"]
;;
(defun lin-to-db (lin)
  (* 20 (/ (log (float lin)) (log 10.0))))

;;; decibel-to-linear conversion
;;
;; NOTE: The function always returns zero if the "reference" is zero.
;;       This is correct because amplified silence remains silence.
;;
;; lin = (10^(db/20))*reference  [general formula]
;;
;; lin = (exp(log(db/20)*10))*reference  [using XLISP "exp" and "log"]
;;
(defun linear-ratio (reference decibel)
  (* (exp (* (log 10.0) (/ decibel 20.0))) reference))

;; using XLISP "expt"
;;
(defun linear-ratio (reference decibel)
  (* (expt 10.0 (/ decibel 20.0)) reference))

;;; db-to-lin - same as Nyquist's db-to-linear [but slower]
;;
;; reference = 1.0   [= soundcard limit, defined by Nyquist]
;; lin = 10^(db/20)  [valid in Nyquist only]
;;
;; lin = exp(log(db/20))  [using XLISP "exp" and "log"]
;;
(defun db-to-lin (db)
  (exp (* (log 10.0) (/ db 20.0))))

;; db-to-lin - using XLISP "expt"
;;
(defun db-to-lin (db)
  (expt 10.0 (/ db 20.0)))

;;;; ------------------------ Pitch and Midi -------------------------

;; pitch values (notes and midi) are based on log2 because 1 octave has
;; a frequency relationship of 2/1.

;; In equal temperament tuning an octave is devidied into 12 steps (notes)
;; with a frequency relationship of 12th-root-of-2/1.

;; see http://en.wikipedia.org/wiki/Music_and_mathematics
;;     http://en.wikipedia.org/wiki/Musical_Instrument_Digital_Interface

;; The Nyquist note names and pitch values are defined at the beginning
;; of the "nyquist.lsp" file in the Audacity "nyquist" directory.

;; The relationship between pitch, midi and frequency is defined as:
;;
;;  note | midi step | frequency
;;   A4  =     69    =   440 Hz

;;; hz-to-pitch - same as Nyquist's "hz-to-step" [but slower]
;;
;; pitch = 69+(12*log2(frequency/440))  [general formula]
;;
;; pitch = 69+(12*(log(frequency/440)/log(2)))  [using XLISP "log"]
;;
(defun hz-to-pitch (frequency)
  (+ 69 (* 12 (/ (log (/ frequency 440.0)) (log 2.0)))))

;;; pitch-to-hz - same as Nyquist's "step-to-hz" [but slower]
;;
;; f = (2^((pitch-69)/12))*440  [general formula]
;;
;; f = exp(log(2)*((pitch-69)/12))*440  [using XLISP "exp" and "log"]
;;
(defun pitch-to-hz (pitch)
  (* (exp (* (log 2.0) (/ (- pitch 69) 12.0))) 440))

;;; pitch-to-hz - using XLISP "expt"
;;
(defun pitch-to-hz (pitch)
  (* (expt 2.0 (/ (- pitch 69) 12.0)) 440))